/* "eqcs_gate.cc"
 * Copyright (C) 1999 Peter Belkner <pbelkner@berlin.snafu.de>
 *
 * This file is part of the eqcs library. This library is free
 * software; you can redistribute it and/or modify it under the terms of
 * the GNU General Public License as published by the Free Software
 * Foundation; either version 2, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this library; see the file COPYING.  If not, write to the
 * Free Software Foundation, 59 Temple Place - Suite 330,
 * Boston, MA  02111-1307, USA.
 */
#ifndef __GNUG__
#pragma implementation
#endif

#include "eqcs_barenco.h"

/*****************************************************************************\
*	quantum coin flip (qcf) gate											  *
\*****************************************************************************/

// See "http://www.sigmaxi.org/amsci/issues/comsci95/compsci95-07.html"
EqcsQcfGate::EqcsQcfGate(int bit) : EqcsMatrix(1, bit)
{
	(*this)[0][0] = sqrt(0.5);
	(*this)[0][1] = -sqrt(0.5);
	(*this)[1][0] = sqrt(0.5);
	(*this)[1][1] = sqrt(0.5);
}

/*****************************************************************************\
*	The Barenco et.al. catalogue (quant-ph/9503016)							  *
\*****************************************************************************/

EqcsBarenco::Rx::Rx(double theta, int bit) : EqcsMatrix(1, bit)
{
	(*this)[0][0] = cos(theta/2);
	(*this)[0][1] = sin(theta/2);
	(*this)[1][0] = -sin(theta/2);
	(*this)[1][1] = cos(theta/2);
}

EqcsBarenco::Rz::Rz(double alpha, int bit) : EqcsMatrix(1, bit)
{
	(*this)[0][0] = polar(1.0, alpha/2);
	(*this)[1][1] = polar(1.0, -alpha/2);
}

EqcsBarenco::Ph::Ph(double delta, int bit) : EqcsMatrix(1, bit)
{
	(*this)[0][1] = polar(1.0, delta);
	(*this)[1][0] = polar(1.0, delta);
}

EqcsBarenco::SigmaX::SigmaX(int bit) : EqcsMatrix(1, bit)
{
	(*this)[0][1] = 1;
	(*this)[1][0] = 1;
}

EqcsBarenco::Identity::Identity(int bit) : EqcsMatrix(1, bit)
{
	(*this)[0][1] = 1;
	(*this)[1][0] = 1;
}
