/* "eqcs_bitset.cc"
 * Copyright (C) 1999 Peter Belkner <pbelkner@berlin.snafu.de>
 *
 * This file is part of the eqcs library. This library is free
 * software; you can redistribute it and/or modify it under the terms of
 * the GNU General Public License as published by the Free Software
 * Foundation; either version 2, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this library; see the file COPYING. If not, write to the
 * Free Software Foundation, 59 Temple Place - Suite 330,
 * Boston, MA  02111-1307, USA.
 */
#ifdef __GNUG__
#pragma implementation
#endif

#include "eqcs_bitset_rep.h"
#include "eqcs_bitset.h"

/*****************************************************************************\
*	bitset																	  *
\*****************************************************************************/

EqcsBitset::EqcsBitset(rep_t *rep) : base_t(rep)
	{}

EqcsBitset::EqcsBitset() : base_t(new rep_t())
	{}

EqcsBitset::EqcsBitset(int nbits) : base_t(new rep_t(nbits))
	{}

void EqcsBitset::set(int i) const
	{ rep()->set(i); }

void EqcsBitset::reset(int i) const
	{ rep()->reset(i); }

bool EqcsBitset::test(int i) const
	{ return rep()->test(i); }

int EqcsBitset::nbits() const
	{ return rep()->nbits(); }

//EqcsBitset::operator bool() const
	//{ return 0 != rep()->nbitson(); }

EqcsBitset EqcsBitset::operator~() const
	{ return rep()->not(); }

EqcsBitset &EqcsBitset::operator&=(const EqcsBitset &bitset)
	{ return *this = rep()->and(bitset); }

EqcsBitset &EqcsBitset::operator|=(const EqcsBitset &bitset)
	{ return *this = rep()->or(bitset); }

EqcsBitset &EqcsBitset::operator<<=(int i)
	{ return *this = rep()->lsh(i); }

EqcsBitset &EqcsBitset::operator>>=(int i)
	{ return *this = rep()->rsh(i); }

int EqcsBitset::leftmost(unsigned long bits)
{
	// We search for the leftmost bit. Instead of simply doing a linear
	// search we should apply binary search.
	unsigned long mask = 1 << (EqcsBitsetRep::BITS_PER_SLOT - 1);

	for (int n = EqcsBitsetRep::BITS_PER_SLOT; 0 < n; --n)
		if (mask & bits)
			return n;
		else
			mask >>= 1;

	return 0;
}

EqcsBitset EqcsBitset::bits2bitset(unsigned long bits)
{
	int nbits = leftmost(bits);

	if (0 == nbits)
		nbits = 1;

	EqcsBitset bitset(nbits);

	for (int i = 0; i < bitset.nbits(); ++i)
		if (bits & (1ul << i))
			bitset.set(i);

	return bitset;
}

ostream &EqcsBitset::put_ket(ostream &os) const
	{ return rep()->put_ket(os); }

ostream &EqcsBitset::put_ket(ostream &os, int nbits) const
	{ return rep()->put_ket(os, nbits); }

ostream &EqcsBitset::put_bits(ostream &os, int nregs, int regwidth) const
	{ return rep()->put_bits(os, nregs, regwidth); }

ostream &EqcsBitset::put_nums(ostream &os, int nregs, int regwidth) const
	{ return rep()->put_nums(os, nregs, regwidth); }

bool operator==(const EqcsBitset &l, const EqcsBitset &r)
	{ return l->cmp(r) == 0; }

bool operator!=(const EqcsBitset &l, const EqcsBitset &r)
	{ return l->cmp(r) != 0; }

bool operator<(const EqcsBitset &l, const EqcsBitset &r)
	{ return l->cmp(r) < 0; }

bool operator<=(const EqcsBitset &l, const EqcsBitset &r)
	{ return l->cmp(r) <= 0; }

bool operator>=(const EqcsBitset &l, const EqcsBitset &r)
	{ return l->cmp(r) >= 0; }

bool operator>(const EqcsBitset &l, const EqcsBitset &r)
	{ return l->cmp(r) > 0; }

EqcsBitset operator&(const EqcsBitset &l, const EqcsBitset &r)
	{ return l->and(r); }

EqcsBitset operator|(const EqcsBitset &l, const EqcsBitset &r)
	{ return l->or(r); }

EqcsBitset operator>>(const EqcsBitset &l, int r)
	{ return l->rsh(r); }

EqcsBitset operator<<(const EqcsBitset &l, int r)
	{ return l->lsh(r); }

ostream &operator<<(ostream &os, const EqcsBitset &bitset)
	{ return bitset->put(os); }

EqcsBitset::omanip0_t ket(const EqcsBitset &bitset)
	{ return EqcsBitset::omanip0_t(bitset, &EqcsBitset::put_ket); }

EqcsBitset::omanip1_t ket(const EqcsBitset &bitset, int nbits)
	{ return EqcsBitset::omanip1_t(bitset, &EqcsBitset::put_ket, nbits); }

EqcsBitset::omanip2_t bits(const EqcsBitset &bitset, int n, int width)
	{ return EqcsBitset::omanip2_t(bitset, &EqcsBitset::put_bits, n, width); }

EqcsBitset::omanip2_t nums(const EqcsBitset &bitset, int n, int width)
	{ return EqcsBitset::omanip2_t(bitset, &EqcsBitset::put_nums, n, width); }
