/* "eqcs_bitset_rep.h"
 * Copyright (C) 1999 Peter Belkner <pbelkner@berlin.snafu.de>
 *
 * This file is part of the eqcs library. This library is free
 * software; you can redistribute it and/or modify it under the terms of
 * the GNU General Public License as published by the Free Software
 * Foundation; either version 2, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this library; see the file COPYING.  If not, write to the
 * Free Software Foundation, 59 Temple Place - Suite 330,
 * Boston, MA  02111-1307, USA.
 */
#ifndef	_EQCS_BITSET_REP_H_
#ifdef __GNUG__
#pragma interface
#endif
#define	_EQCS_BITSET_REP_H_

#include "eqcs_body.h"

#include <vector>

class ostream;

/*****************************************************************************\
*	bitset representation													  *
\*****************************************************************************/

class EqcsBitsetRep: public EqcsBody {
public:
	typedef unsigned long slot_t;
	typedef vector<slot_t> rep_t;

	enum {
		BITS_PER_BYTE = 8,
		BITS_PER_SLOT = BITS_PER_BYTE * sizeof(slot_t)
	};

	EqcsBitsetRep(int nbits=BITS_PER_SLOT);
	virtual ~EqcsBitsetRep();

	int nbits() const;
	int nbitson() const;

	void set(int i);
	void reset(int i);
	bool test(int i) const;

	int cmp(const EqcsBitsetRep *bitset) const;
	EqcsBitsetRep *not() const;
	EqcsBitsetRep *and(const EqcsBitsetRep *bitset) const;
	EqcsBitsetRep *or(const EqcsBitsetRep *bitset) const;
	EqcsBitsetRep *rsh(int i) const;
	EqcsBitsetRep *lsh(int i) const;

	ostream &put_bit(ostream &os, int i) const;
	ostream &put(ostream &os) const;
	ostream &put_ket(ostream &os) const;
	ostream &put_ket(ostream &os, int nbits) const;
	ostream &put_bits(ostream &os, int nregs, int regwidth) const;
	ostream &put_nums(ostream &os, int nregs, int regwidth) const;
private:
	static int dim(int nbits);

	int		_nbits;			// The number of bits.
	int		_nbitson;		// The number of bis set on.
	rep_t	_rep;
};

#endif	//	_EQCS_BITSET_REP_H_
