/* "eqcs_gate_rep.h"
 * Copyright (C) 1999 Peter Belkner <pbelkner@berlin.snafu.de>
 *
 * This file is part of the eqcs library. This library is free
 * software; you can redistribute it and/or modify it under the terms of
 * the GNU General Public License as published by the Free Software
 * Foundation; either version 2, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this library; see the file COPYING.  If not, write to the
 * Free Software Foundation, 59 Temple Place - Suite 330,
 * Boston, MA  02111-1307, USA.
 */
#ifndef	_EQCS_GATE_REP_H_
#ifdef __GNUG__
#pragma interface
#endif
#define	_EQCS_GATE_REP_H_

#include <complex>
#include <vector>
#include "eqcs_state_rep.h"

class ostream;
class EqcsStateRep;

/*****************************************************************************\
*	gate representation														  *
\*****************************************************************************/

class EqcsGateRep: public EqcsBody {
public:
	typedef EqcsStateRep::bitset_t bitset_t;
	typedef EqcsStateRep::complex_t complex_t;
	typedef vector<int> args_t;

	EqcsGateRep(int nqubits);
	virtual ~EqcsGateRep();

	virtual EqcsStateRep *apply(const EqcsStateRep *state) = 0;
	virtual ostream &put(ostream &os) const = 0;

	inline unsigned long dim() const
		{ return 1ul << _args.size(); }
	inline int maxbit() const
		{ return _maxbit; }
	const bitset_t &bits() const
		{ return _bits; }

	void set(int i, int bit);
	int bitset2index(const bitset_t &bitset) const;
	bitset_t index2bitset(int index) const;
private:
	int			_maxbit;		// The leftmost argument bit.
	args_t		_args;			// The argument bits.
	bitset_t	_bits;
};

#endif	//	_EQCS_GATE_REP_H_
