/* "eqcs_vedral.cc"
 * Copyright (C) 1999 Peter Belkner <pbelkner@berlin.snafu.de>
 *
 * This file is part of the eqcs library. This library is free
 * software; you can redistribute it and/or modify it under the terms of
 * the GNU General Public License as published by the Free Software
 * Foundation; either version 2, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this library; see the file COPYING.  If not, write to the
 * Free Software Foundation, 59 Temple Place - Suite 330,
 * Boston, MA  02111-1307, USA.
 */
#ifdef __GNUG__
#pragma implementation
#endif

#include "eqcs_vedral.h"

/*****************************************************************************\
*	Vedral et.al. arithmetics (quant-ph/9511018)							  *
\*****************************************************************************/

EqcsVedral::Not::Not(int bit) : EqcsLambda(0.0, 1.0, 1.0, 0.0, 0)
	{ set(0, bit); }

EqcsVedral::CNot::CNot(int bit, int c) : EqcsLambda(0.0, 1.0, 1.0, 0.0, 1)
{
	set(0, bit);
	set(1, c);
}

EqcsVedral::CCNot::CCNot(int bit, int c1, int c0) :
	EqcsLambda(0.0, 1.0, 1.0, 0.0, 2)
{
	set(0, bit);
	set(1, c1);
	set(2, c0);
}

void EqcsVedral::carry(EqcsGateArray &a, int b0, int b1, int b2, int b3)
{
	a.push_back(CCNot(b0, b1, b2));
	a.push_back(CNot(b1, b2));
	a.push_back(CCNot(b0, b1, b3));
}

void EqcsVedral::rcarry(EqcsGateArray &a, int b0, int b1, int b2, int b3)
{
	a.push_back(CCNot(b0, b1, b3));
	a.push_back(CNot(b1, b2));
	a.push_back(CCNot(b0, b1, b2));
}

void EqcsVedral::sum(EqcsGateArray &a, int b0, int b1, int b2)
{
	a.push_back(CNot(b0, b1));
	a.push_back(CNot(b0, b2));
}

void EqcsVedral::rsum(EqcsGateArray &a, int b0, int b1, int b2)
{
	a.push_back(CNot(b0, b2));
	a.push_back(CNot(b0, b1));
}

void EqcsVedral::plain_adder(EqcsGateArray &a, int n)
{
	int a0 = 0;
	int b0 = n;
	int c0 = 2 * n;

	--n;

	for (int i = 0; i < n; ++i)
		carry(a, c0 + 1 + i, b0 + i, a0 + i, c0 + i);

	carry(a, b0 + n - 1, b0 + n, a0 + n, c0 + n);
	a.push_back(CNot(b0 + n, a0 + n));
	sum(a, b0 + n, a0 + n, c0 + n);

	for (int i = n - 1; i >= 0; --i) {
		rcarry(a, c0 + i + 1, b0 + i, a0 + i, c0 + i);
		sum(a, b0 + i, a0 + i, c0 + i);
	}
}

void EqcsVedral::rplain_adder(EqcsGateArray &a, int n)
{
	int a0 = 0;
	int b0 = n;
	int c0 = 2 * n;

	--n;

	for (int i = 0; i < n; ++i) {
		sum(a, b0 + i, a0 + i, c0 + i);
		rcarry(a, c0 + i + 1, b0 + i, a0 + i, c0 + i);
	}

	rsum(a, b0 + n, a0 + n, c0 + n);
	a.push_back(CNot(b0 + n, a0 + n));
	rcarry(a, b0 + n - 1, b0 + n, a0 + n, c0 + n);

	for (int i = n - 1; i >= 0; --i)
		rcarry(a, c0 + 1 + i, b0 + i, a0 + i, c0 + i);
}
