/* "eqcs_vedral.h"
 * Copyright (C) 1999 Peter Belkner <pbelkner@berlin.snafu.de>
 *
 * This file is part of the eqcs library. This library is free
 * software; you can redistribute it and/or modify it under the terms of
 * the GNU General Public License as published by the Free Software
 * Foundation; either version 2, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this library; see the file COPYING.  If not, write to the
 * Free Software Foundation, 59 Temple Place - Suite 330,
 * Boston, MA  02111-1307, USA.
 */
#ifndef	_EQCS_VEDRAL_H_
#ifdef __GNUG__
#pragma interface
#endif
#define	_EQCS_VEDRAL_H_

#include "eqcs_gate_array.h"
#include "eqcs_lambda.h"

/*****************************************************************************\
*	Vedral et.al. arithmetics (quant-ph/9511018)							  *
\*****************************************************************************/

class EqcsVedral {
public:
	// The not gate.
	class Not: public EqcsLambda {
	public:
    	Not(int bit);
	};

	// The controlled not gate.
	class CNot: public EqcsLambda {
	public:
    	CNot(int c, int bit);
	};

	// The controlled controlled not gate.
	class CCNot: public EqcsLambda {
	public:
    	CCNot(int c0, int c1, int bit);
	};

	// Elementary carry network.
	static void carry(EqcsGateArray &a, int b0, int b1, int b2, int b3);
	// Elementary carry network in reversed order.
	static void rcarry(EqcsGateArray &a, int b0, int b1, int b2, int b3);
	// Elementary sum network.
	static void sum(EqcsGateArray &a, int b0, int b1, int b2);
	// Elementary sum network in reversed order.
	static void rsum(EqcsGateArray &a, int b0, int b1, int b2);

	// Plain adder network for adding two n-bit numbers.
	static void plain_adder(EqcsGateArray &a, int n);
	// Plain adder network for adding two n-bit numbers in reversed order.
	static void rplain_adder(EqcsGateArray &a, int n);
};

#endif	//	_EQCS_VEDRAL_H_
